pathfind = {}
pathfind.open = {}
pathfind.closed = {}
nodes = {}

function pathfind.generateNodes()
	if #nodes > 0 then
		pathfind.clearNodes()
		nodes = {}
	end
	for y = 1, #map do
		nodes[y] = {}
		for x = 1, #map[y] do
			nodes[y][x] = {
				x = map[y][x].x + 16,
				y = map[y][x].y + 16,
				collidable = map[y][x].collidable
			}
		end
	end
end

function pathfind.clearNodes()
	for y = 1, #nodes do
		table.remove(nodes[y])
	end
end

local function hcost(v1, v2)
	return distV(v1, v2)
end

local function backTrack(node)
	local path = {}
	repeat
		table.insert(path, 1, node)
		node = node.parent
	until not node
	return path
end

function pathfind.draw()
	gr.setColor(255, 0, 0, 255)
	for k,v in ipairs(pathfind.open) do
		gr.circle("fill", v.x, v.y, 5, 50)
	end
	gr.setColor(0, 255, 0, 255)
	for k,v in ipairs(pathfind.closed) do
		gr.circle("fill", v.x, v.y, 5, 50)
	end
end

function pathfind.findPath(goal)
	for y = 1, #nodes do
		for x = 1, #nodes[y] do
			if boxCollision(nodes[y][x].x - 5, nodes[y][x].y - 5, 10, 10, player.x, player.y, player.w, player.h) then
				nodes[y][x].gcost = 0
				nodes[y][x].hcost = hcost(nodes[y][x], goal)
				nodes[y][x].fcost = nodes[y][x].gcost + nodes[y][x].hcost
				nodes[y][x].parent = {}
				table.insert(pathfind.open, nodes[y][x])
				
				if not nodes[y+1][x].collidable then
					nodes[y+1][x].parent = nodes[y][x]
					nodes[y+1][x].hcost = hcost(nodes[y+1][x], goal)
					nodes[y+1][x].gcost = nodes[y+1][x].parent.gcost + distV(nodes[y+1][x], nodes[y][x])
					nodes[y+1][x].fcost = nodes[y+1][x].gcost + nodes[y+1][x].hcost
					table.insert(pathfind.open, nodes[y+1][x])
				end
				if not nodes[y-1][x].collidable then
					nodes[y-1][x].parent = nodes[y][x]
					nodes[y-1][x].hcost = hcost(nodes[y-1][x], goal)
					nodes[y-1][x].gcost = nodes[y-1][x].parent.gcost + distV(nodes[y-1][x], nodes[y][x])
					nodes[y-1][x].fcost = nodes[y-1][x].gcost + nodes[y-1][x].hcost
					table.insert(pathfind.open, nodes[y-1][x])
				end
				if not nodes[y][x+1].collidable then
					nodes[y][x+1].parent = nodes[y][x]
					nodes[y][x+1].hcost = hcost(nodes[y][x+1], goal)
					nodes[y][x+1].gcost = nodes[y][x+1].parent.gcost + distV(nodes[y][x+1], nodes[y][x])
					nodes[y][x+1].fcost = nodes[y][x+1].gcost + nodes[y][x+1].hcost
					table.insert(pathfind.open, nodes[y][x+1])
				end
				if not nodes[y][x-1].collidable then
					nodes[y][x-1].parent = nodes[y][x]
					nodes[y][x-1].hcost = hcost(nodes[y][x-1], goal)
					nodes[y][x-1].gcost = nodes[y][x-1].parent.gcost + distV(nodes[y][x-1], nodes[y][x])
					nodes[y][x-1].fcost = nodes[y][x-1].gcost + nodes[y][x-1].hcost
					table.insert(pathfind.open, nodes[y][x-1])
				end
				if not nodes[y-1][x+1].collidable then
					nodes[y-1][x+1].parent = nodes[y][x]
					nodes[y-1][x+1].hcost = hcost(nodes[y-1][x+1], goal)
					nodes[y-1][x+1].gcost = nodes[y-1][x+1].parent.gcost + distV(nodes[y-1][x+1], nodes[y][x])
					nodes[y-1][x+1].fcost = nodes[y-1][x+1].gcost + nodes[y-1][x+1].hcost
					table.insert(pathfind.open, nodes[y-1][x+1])
				end
				if not nodes[y+1][x-1].collidable then
					nodes[y+1][x-1].parent = nodes[y][x]
					nodes[y+1][x-1].hcost = hcost(nodes[y+1][x-1], goal)
					nodes[y+1][x-1].gcost = nodes[y+1][x-1].parent.gcost + distV(nodes[y+1][x-1], nodes[y][x])
					nodes[y+1][x-1].fcost = nodes[y+1][x-1].gcost + nodes[y+1][x-1].hcost
					table.insert(pathfind.open, nodes[y+1][x-1])
				end
				if not nodes[y+1][x+1].collidable then
					nodes[y+1][x+1].parent = nodes[y][x]
					nodes[y+1][x+1].hcost = hcost(nodes[y+1][x+1], goal)
					nodes[y+1][x+1].gcost = nodes[y+1][x+1].parent.gcost + distV(nodes[y+1][x+1], nodes[y][x])
					nodes[y+1][x+1].fcost = nodes[y+1][x+1].gcost + nodes[y+1][x+1].hcost
					table.insert(pathfind.open, nodes[y+1][x+1])
				end
				if not nodes[y-1][x-1].collidable then
					nodes[y-1][x-1].parent = nodes[y][x]
					nodes[y-1][x-1].hcost = hcost(nodes[y-1][x-1], goal)
					nodes[y-1][x-1].gcost = nodes[y-1][x-1].parent.gcost + distV(nodes[y-1][x-1], nodes[y][x])
					nodes[y-1][x-1].fcost = nodes[y-1][x-1].gcost + nodes[y-1][x-1].hcost
					table.insert(pathfind.open, nodes[y-1][x-1])
				end
			end
		end
	end
	
	while #pathfind.open > 0 do
		local currentNodeId = 1
		for i = 1, #pathfind.open do
			if pathfind.open[i].fcost < pathfind.open[currentNodeId].fcost then
				currentNodeId = i
			end
		end
		
		local currentNode = table.remove(pathfind.open, currentNodeId)
		table.insert(pathfind.closed, currentNode)
		
		if currentNode.x == goal.x and currentNode.y == goal.y then
			for i = 1, #pathfind.open do
				table.remove(pathfind.open, i)
			end
			for i = 1, #pathfind.closed do
				table.remove(pathfind.closed, i)
			end
			return backTrack(currentNode)
		end
		
		for y = 1, #nodes do
			for x = 1, #nodes[y] do
				if nodes[y][x].x == currentNode.x and nodes[y][x].y == currentNode.y then
					if not nodes[y+1][x].collidable and nodes[y+1][x].parent == nil then
						nodes[y+1][x].parent = nodes[y][x]
						nodes[y+1][x].hcost = hcost(nodes[y+1][x], goal)
						nodes[y+1][x].gcost = nodes[y+1][x].parent.gcost + distV(nodes[y+1][x], nodes[y][x])
						nodes[y+1][x].fcost = nodes[y+1][x].gcost + nodes[y+1][x].hcost
						table.insert(pathfind.open, nodes[y+1][x])
					end
					if not nodes[y-1][x].collidable and nodes[y-1][x].parent == nil then
						nodes[y-1][x].parent = nodes[y][x]
						nodes[y-1][x].hcost = hcost(nodes[y-1][x], goal)
						nodes[y-1][x].gcost = nodes[y-1][x].parent.gcost + distV(nodes[y-1][x], nodes[y][x])
						nodes[y-1][x].fcost = nodes[y-1][x].gcost + nodes[y-1][x].hcost
						table.insert(pathfind.open, nodes[y-1][x])
					end
					if not nodes[y][x+1].collidable and nodes[y][x+1].parent == nil then
						nodes[y][x+1].parent = nodes[y][x]
						nodes[y][x+1].hcost = hcost(nodes[y][x+1], goal)
						nodes[y][x+1].gcost = nodes[y][x+1].parent.gcost + distV(nodes[y][x+1], nodes[y][x])
						nodes[y][x+1].fcost = nodes[y][x+1].gcost + nodes[y][x+1].hcost
						table.insert(pathfind.open, nodes[y][x+1])
					end
					if not nodes[y][x-1].collidable and nodes[y][x-1].parent == nil then
						nodes[y][x-1].parent = nodes[y][x]
						nodes[y][x-1].hcost = hcost(nodes[y][x-1], goal)
						nodes[y][x-1].gcost = nodes[y][x-1].parent.gcost + distV(nodes[y][x-1], nodes[y][x])
						nodes[y][x-1].fcost = nodes[y][x-1].gcost + nodes[y][x-1].hcost
						table.insert(pathfind.open, nodes[y][x-1])
					end
					if not nodes[y-1][x+1].collidable and nodes[y-1][x+1].parent == nil then
						nodes[y-1][x+1].parent = nodes[y][x]
						nodes[y-1][x+1].hcost = hcost(nodes[y-1][x+1], goal)
						nodes[y-1][x+1].gcost = nodes[y-1][x+1].parent.gcost + distV(nodes[y-1][x+1], nodes[y][x])
						nodes[y-1][x+1].fcost = nodes[y-1][x+1].gcost + nodes[y-1][x+1].hcost
						table.insert(pathfind.open, nodes[y-1][x+1])
					end
					if not nodes[y+1][x-1].collidable and nodes[y+1][x-1].parent == nil then
						nodes[y+1][x-1].parent = nodes[y][x]
						nodes[y+1][x-1].hcost = hcost(nodes[y+1][x-1], goal)
						nodes[y+1][x-1].gcost = nodes[y+1][x-1].parent.gcost + distV(nodes[y+1][x-1], nodes[y][x])
						nodes[y+1][x-1].fcost = nodes[y+1][x-1].gcost + nodes[y+1][x-1].hcost
						table.insert(pathfind.open, nodes[y+1][x-1])
					end
					if not nodes[y+1][x+1].collidable and nodes[y+1][x+1].parent == nil then
						nodes[y+1][x+1].parent = nodes[y][x]
						nodes[y+1][x+1].hcost = hcost(nodes[y+1][x+1], goal)
						nodes[y+1][x+1].gcost = nodes[y+1][x+1].parent.gcost + distV(nodes[y+1][x+1], nodes[y][x])
						nodes[y+1][x+1].fcost = nodes[y+1][x+1].gcost + nodes[y+1][x+1].hcost
						table.insert(pathfind.open, nodes[y+1][x+1])
					end
					if not nodes[y-1][x-1].collidable and nodes[y-1][x-1].parent == nil then
						nodes[y-1][x-1].parent = nodes[y][x]
						nodes[y-1][x-1].hcost = hcost(nodes[y-1][x-1], goal)
						nodes[y-1][x-1].gcost = nodes[y-1][x-1].parent.gcost + distV(nodes[y-1][x-1], nodes[y][x])
						nodes[y-1][x-1].fcost = nodes[y-1][x-1].gcost + nodes[y-1][x-1].hcost
						table.insert(pathfind.open, nodes[y-1][x-1])
					end
				end
			end
		end
	end
	
	
	
	
	
	
	
	
	
end